﻿#ifndef RP_PROCESS_LOG_CPP_ASYNCER_H
#define RP_PROCESS_LOG_CPP_ASYNCER_H

#include <deque>
#include <chrono>
#include <mutex>
#include <thread>
#include <condition_variable>
#include <rp_process_log/process_log_define.h>
#include <rp_process_log/process_log_cpp_base.h>

namespace rp_process_log
{

CLASS_FORWARD( ProcessLogCppAsyncer )

/**
 *  @class  ProcessLogCppAsyncer
 *  @class  ProcessLogCppAsyncer
 *  @brief  异步日志处理类
 *  @author luyp
 */
class ProcessLogCppAsyncer : public ProcessLogCppBase
{
 public:
  ProcessLogCppAsyncer( const char *module );
  virtual ~ProcessLogCppAsyncer();

  /**
   *  @brief 设置参数
   *  @param [in] key 参数名
   *  @param [in] val 参数值
   */
  virtual void setConfig( const char *key, const char *val ) override;
  /**
   *  @brief 初始化
   *  @param [in] param 参数名
   *  @param [in] val 参数值
   *  @return 返回是否成功
   */
  virtual bool init( const char *param ) override;
  /**
   *  @brief 日志处理
   *  @param [in] log_record 参数名
   *  @param [in] val 参数值
   *  @return 返回是否成功
   */
  virtual bool process( LogRecordInfoPtr log_record ) override;

 protected:
  /**
   *  @brief 异步处理线程
   */
  void threadWork();
  std::deque<LogRecordInfoPtr> log_deque_;
  std::shared_ptr<std::thread> thread_;
  std::mutex mutex_;
  std::condition_variable condition_;
  bool running_;
};

}  // namespace rp_process_log

#endif  // RP_PROCESS_LOG_CPP_ASYNCER_H
