#ifndef RP_PROCESS_LOG_CPP_BASE_H
#define RP_PROCESS_LOG_CPP_BASE_H

#include <chrono>
#include <rp_process_log/process_log_define.h>


namespace rp_process_log {

//日志级别
enum LogLevel
{
  LV_NULL = 0,
  LV_FATAL,
  LV_ERROR,
  LV_WARN,
  LV_INFO,
  LV_DEBUG,
  LV_ALL
};

#define LogTimePoint std::chrono::system_clock::time_point

CLASS_FORWARD(LogRecordInfo)
CLASS_FORWARD(ProcessLogCppBase)

/**
*  @class LogRecordInfo
*  @brief brief 日志信息暂存类
*  @author luyp
*/
class LogRecordInfo
{
public:
  std::string  key;
  LogLevel     level;
  LogTimePoint time;
  std::string  log;
  std::string  filename;
  uint32_t     line;
  std::string  funcname;

  /**
  *  @brief 清理数据
  */
  void clear()
  {
    key.clear();
    level = LV_NULL;
    time  = std::chrono::system_clock::now();
    log.clear();
    filename.clear();
    line = 0;
  }
};

/**
*  @class ProcessLogCppBase
*  @brief brief 日志处理基类
*  @author luyp
*/
class ProcessLogCppBase
{
public:
    /**
    *  @brief 构造函数
    *  @param [in] module 模块名
    */
    ProcessLogCppBase(const char* module){module_name_ = module;}
    virtual ~ProcessLogCppBase() = default;
    /**
    *  @brief 初始化
    *  @param [in] param 参数名
    *  @param [in] val 参数值
    *  @return 返回是否成功
    */
    virtual bool init(const char*){return true;};
    /**
    *  @brief 日志处理
    *  @param [in] log_record 参数名
    *  @param [in] val 参数值
    *  @return 返回是否成功
    */
    virtual bool process(LogRecordInfoPtr info)
    {
        if(fn_)
        {
            fn_(info);
        }
        return true;
    }
    /**
    *  @brief 设置参数
    *  @param [in] key 参数名
    *  @param [in] val 参数值
    */
    virtual void setConfig(const char* key, const char* val){}
    /**
    *  @brief 设置后续处理回调函数
    *  @param [in] key 关键字，目前不用
    *  @param [in] fn  回调函数
    */
    virtual void addNextHandle(const char* key,std::function<void(LogRecordInfoPtr)> fn){fn_ = fn;}
protected:
    std::function<void(LogRecordInfoPtr)> fn_;
    std::string module_name_;
};

}

#endif  // RP_PROCESS_LOG_CPP_BASE_H
