#ifndef RP_PROCESS_LOG_CPP_FILTER_H
#define RP_PROCESS_LOG_CPP_FILTER_H

#include <mutex>
#include <chrono>
#include <set>
#include <rp_process_log/process_log_define.h>
#include <rp_process_log/process_log_cpp_base.h>

namespace rp_process_log {

//设置过滤日志级别，空为不过滤
#define CONFIG_KEY_LEVEL_PASS  "level"
#define CONFIG_VAL_LEVEL_NONE  "NONE"
#define CONFIG_VAL_LEVEL_FATAL "FATAL"
#define CONFIG_VAL_LEVEL_ERROR "ERROR"
#define CONFIG_VAL_LEVEL_WARN  "WARN"
#define CONFIG_VAL_LEVEL_INFO  "INFO"
#define CONFIG_VAL_LEVEL_DEBUG "DEBUG"

//设置关键字过滤，空为不过滤
#define CONFIG_KEY_FILTER_KEY      "key"
//设置源文件过滤，空为不过滤
#define CONFIG_KEY_FILTER_SRC_FILE "source_file"

CLASS_FORWARD(ProcessLogCppFilter)

/**
*  @class ProcessLogCppFilter
*  @brief brief 日志处理过滤类
*  @author luyp
*/
class ProcessLogCppFilter : public ProcessLogCppBase
{
public:
    ProcessLogCppFilter(const char* module);
    virtual ~ProcessLogCppFilter() = default;

    virtual bool init(const char*) override;
    virtual bool process(LogRecordInfoPtr) override;
    virtual void setConfig(const char* key, const char* val) override;

protected:
    LogLevel level_;
    //std::string filter_key_;
    std::string filter_file_;
    std::set<std::string> forward_filter_keys_;
    std::set<std::string> reverse_filter_keys_;
};

}

#endif  // RP_PROCESS_LOG_CPP_FILTER_H
