#include <stdio.h>
#include <string.h>
#include <chrono>
#include <iostream>
#include <vector>
#include <map>
#include <thread>
#include <rp_process_log/process_log_define.h>
#include <rp_process_log/process_log_cpp_asyncer.h>

namespace rp_process_log
{

// REGIST_RP_PROCESS_LOG_MODULE("asyncer", ProcessLogCppAsyncer)

ProcessLogCppAsyncer::ProcessLogCppAsyncer( const char *module )
    : ProcessLogCppBase( module ), running_( false )
{
}

ProcessLogCppAsyncer::~ProcessLogCppAsyncer()
{
  running_ = false;
  condition_.notify_all();
  if ( thread_->joinable() )
  {
    thread_->join();
  }
}

void ProcessLogCppAsyncer::threadWork()
{
  while ( running_ )
  {
    int que_size = 0;
    {
      std::unique_lock<std::mutex> lk( mutex_ );
      condition_.wait( lk, [ = ]() { return !running_ || ( running_ && !!log_deque_.size() ); } );
      que_size = log_deque_.size();
    }
#if 1
    while ( que_size-- > 0 )
    {
      LogRecordInfoPtr msg = nullptr;
      {
        std::unique_lock<std::mutex> lk( mutex_ );
        msg = log_deque_.front();
        log_deque_.pop_front();
      }
      if ( msg )
      {
        ProcessLogCppBase::process( msg );
      }
    }
#else
    std::vector<LogRecordInfoPtr> msgs;
    while ( que_size-- > 0 )
    {
      std::unique_lock<std::mutex> lk( mutex_ );
      LogRecordInfoPtr msg = nullptr;
      msg                  = log_deque_.front();
      log_deque_.pop_front();
      msgs.push_back( msg );
    }
    for ( auto msg : msgs )
    {
      ProcessLogCppBase::process( msg );
    }
#endif
  }
}

bool ProcessLogCppAsyncer::init( const char * )
{
  running_ = true;
  thread_  = std::make_shared<std::thread>( &ProcessLogCppAsyncer::threadWork, this );
  return true;
}

bool ProcessLogCppAsyncer::process( LogRecordInfoPtr info )
{
  {
    std::unique_lock<std::mutex> lk( mutex_ );
    log_deque_.push_back( info );
    condition_.notify_one();
  }
  return true;
}

void ProcessLogCppAsyncer::setConfig( const char *key, const char *val )
{
  UNUSE( key );
  UNUSE( val );
}

}  // namespace rp_process_log
