#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <platform_api/common/platform_unistd.h>
//#include <unistd.h>
#include <string.h>
#include <chrono>
#include <iostream>
#include <fstream>
#include <vector>
#include <map>
#include <rp_process_log/process_log_define.h>
#include <rp_process_log/process_log_cpp_sink_tofile.h>

namespace rp_process_log {

//REGIST_RP_PROCESS_LOG_MODULE("writer", ProcessLogCppSinkToFile)

constexpr uint32_t day_seconds = 60*60*24;

static int dirAccess(const char* path)
{
    struct stat dir_stat;

    auto ret_code = stat(path, &dir_stat);
    if(ret_code == 0)
    {
        if(dir_stat.st_mode & S_IFDIR)
        {
            return 0;
        }
        else
        {
            std::cerr << path << " is not dir" << std::endl;
            return 1;
        }
    }

    auto error_code = errno;
    if(error_code == ENOTDIR || error_code == EACCES)
    {
        std::cerr << path << " can not be accessed" << std::endl;
        return 1;
    }
    return 1;
}

static bool createPath(const std::string path)
{
//#ifdef __linux
#define PATH_SPLIT "/"
#define PATH_SPLIT_C '/'
//#else
//#define PATH_SPLIT "\\"
//#endif
    size_t start = 0, next = 0;
    if(path.size() && path.at(0) == PATH_SPLIT_C)
    {
        next = start = 1;
    }
    while(path.size() > 1 || start == 0)
    {
        if(next == std::string::npos)
        {
            break;
        }
        next = path.find(PATH_SPLIT, start);
        auto parent_path = path.substr(0, next);

        if(dirAccess(parent_path.c_str()) == 0)
        {
            start = next + strlen(PATH_SPLIT);
            continue;
        }
#ifdef __linux
        if(mkdir(parent_path.c_str(), S_IRWXU | S_IRWXG) != 0)
#endif
#ifdef _WIN32
        if (mkdir(parent_path.c_str()) != 0)
#endif
        {
            if(dirAccess(parent_path.c_str()) == 0)
            {
                continue;
            }
            return false;
        }

        if(next == std::string::npos)
        {
            break;
        }
        start = next + strlen(PATH_SPLIT);
    }
    return true;
}


bool ProcessLogCppSinkToFile::newStampFile()
{

    char nowt[20] = {0};

    strftime(nowt, sizeof(nowt), "%Y-%m-%d", localtime(&time_log_));

    std::string logpath = path_;
    
    logpath += std::string("/") + nowt;

    if (dirAccess(logpath.c_str()) != 0)
    {
#ifdef __linux
        mkdir(logpath.c_str(), S_IRWXU | S_IRWXG);
#endif
#ifdef _WIN32
        mkdir(logpath.c_str());
#endif
    }

    logpath += std::string("/") + module_name_;
    if(dirAccess(logpath.c_str()) != 0)
    {
#ifdef __linux
        mkdir(logpath.c_str(), S_IRWXU | S_IRWXG);
#endif
#ifdef _WIN32
        mkdir(logpath.c_str());
#endif
    }
    /*
    logpath += std::string("/") + nowt;

    if(dirAccess(logpath.c_str()) != 0)
    {
#ifdef __linux
        mkdir(logpath.c_str(), S_IRWXU | S_IRWXG);
#endif
#ifdef _WIN32
        mkdir(logpath.c_str());
#endif
    }
    */
    logpath += "/" + prefix_ + suffix_ + ".log";

    
    out_file_ = std::make_shared<std::ofstream>(logpath, std::ios::app);

    auto ret = !!out_file_;

    if(!ret)
    {
        std::cerr << "fail to open the file: " << logpath <<std::endl;
    }

    return ret;
}

bool ProcessLogCppSinkToFile::checkFile( LogRecordInfoPtr info)
{
    auto tnow = std::chrono::system_clock::to_time_t(info->time);

    if(tnow/day_seconds != time_log_/day_seconds)
    {
        std::lock_guard<std::mutex> lk(mutex_);
        time_log_ = tnow;
        return newStampFile();
    }

    return true;

}


ProcessLogCppSinkToFile::ProcessLogCppSinkToFile(const char* module) :
    ProcessLogCppBase(module)
{
    
}

bool ProcessLogCppSinkToFile::init(const char*)
{
    return true;
}
#ifdef __linux
static std::string expand_user(std::string path) {
  if (!path.empty() && path[0] == '~') {
    if(path.size() == 1 || path[1] == '/')
    {
      // or other error handling
      char const* home = getenv("HOME");
      if (home || (home = getenv("USERPROFILE"))) {
        path.replace(0, 1, home);
      }
      else {
        char const *hdrive = getenv("HOMEDRIVE"),
          *hpath = getenv("HOMEPATH");
        if(hdrive && hpath)
        {
          path.replace(0, 1, std::string(hdrive) + hpath);
        }
      }
    } 

  }
  return path;
}
#endif
void ProcessLogCppSinkToFile::setConfig(const char* key, const char* val)
{
    if(strcmp(key, CONFIG_KEY_DUMP_PATH) == 0)
    {
#ifdef __linux
        path_ = expand_user(val);
#endif
#ifdef _WIN32
        path_ = val;
#endif
        createPath(path_.c_str());
    }
    if(strcmp(key, CONFIG_KEY_DUMP_PREFIX) == 0)
    {
        prefix_ = val;
    }
    if(strcmp(key, CONFIG_KEY_DUMP_SUFFIX) == 0)
    {
        suffix_ = val;
    }
}

bool ProcessLogCppSinkToFile::process(LogRecordInfoPtr info)
{

    std::string  key  = info->key;
    auto         log  = info->log;

    bool ret = true;

    if(checkFile(info))
    {
        auto& out_file = *out_file_;
        std::lock_guard<std::mutex> lk(mutex_ );
        out_file << log;
        out_file.flush();
    }

    return ProcessLogCppBase::process(info);
}



}
