#include <stdio.h>
#include <platform_api/common/platform_unistd.h>
#include <cstdarg>
#include <iostream>
#include <functional>
#include <memory>
#include <atomic>
#include <string.h>
#ifdef __linux
#include <jsoncpp/json/json.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>
#endif

#ifdef _WIN32
#include <windows.h>
#include <json/json.h>
#endif
#include <fstream>
#include <rp_process_log/process_log_define.h>
#include <rp_process_log/process_log_cpp_base.h>
#include <rp_process_log/process_log_cpp_filter.h>
#include <rp_process_log/process_log_cpp_formater.h>
#include <rp_process_log/process_log_cpp_asyncer.h>
#include <rp_process_log/process_log_cpp_sink_tofile.h>
#include <rp_process_log/process_log_cpp_printer.h>
#include <rp_process_log/process_log_cpp_forker.h>
#include <rp_process_log/process_log_frame.h>

#define RP_LOG_CFG ".rp_process_log.cfg"

namespace rp_process_log
{

std::map<std::string, std::function<ProcessLogCppBasePtr( std::string module_name )>> g_module_generators;

std::deque<LogRecordInfo *> g_log_cycle_deque;
std::mutex g_log_cycle_mutex;

static int dirAccess( const char *path )
{
  struct stat dir_stat;

  auto ret_code = stat( path, &dir_stat );
  if ( ret_code == 0 )
  {
    return 0;

    if ( dir_stat.st_mode & S_IFDIR )
    {
      return 0;
    }
    else
    {
      std::cerr << path << " is not dir" << std::endl;
      return 1;
    }
  }

  auto error_code = errno;
  if ( error_code == ENOTDIR || error_code == EACCES )
  {
    std::cerr << path << " can not be accessed" << std::endl;
    return 1;
  }
  return 1;
}

static int FileAccess( const char *path )
{
  struct stat dir_stat;

  auto ret_code = stat( path, &dir_stat );
  if ( ret_code == 0 )
  {
    return 0;
    /*
    if (dir_stat.st_mode & S_IFDIR)
    {
        return 0;
    }
    else
    {
        std::cerr << path << " is not dir" << std::endl;
        return 1;
    }
    */
  }

  auto error_code = errno;
  if ( error_code == ENOTDIR || error_code == EACCES )
  {
    std::cerr << path << " can not be accessed" << std::endl;
    return 1;
  }
  return 1;
}

#ifdef __linux
#define PATH_SPLIT "/"
#endif
#ifdef _WIN32
#define PATH_SPLIT "\\"
#endif

static std::string completePath( std::string path )
{
  if ( path.size() )
  {
    if ( path.find_last_of( PATH_SPLIT ) != path.size() - 1 )
    {
      path += PATH_SPLIT;
    }
  }
  return path;
}

static std::string completePath( std::string &&path )
{
  std::string tmp = std::move( path );
  return completePath( tmp );
}

static std::string getExeName()
{
#define PATH_MAX 4096
#ifdef __linux
  char result[ PATH_MAX ];
  ssize_t count       = readlink( "/proc/self/exe", result, PATH_MAX );
  std::string exePath = std::string( result, ( count > 0 ) ? static_cast<size_t>( count ) : 0 );
  size_t pos          = exePath.find_last_of( "\\/" );
  return ( std::string::npos == pos ) ? exePath : exePath.substr( pos + 1 );
#else
  char result[ PATH_MAX ];
  GetModuleFileNameA( NULL, result, 255 );
  std::string exePath = std::string( result );

  size_t pos = exePath.find_last_of( "\\" );
  return exePath.substr( 0, pos );
  ;
#endif
}

static std::string getExePath()
{
#define PATH_MAX 4096
#ifdef __linux
  char result[ PATH_MAX ];
  ssize_t count       = readlink( "/proc/self/exe", result, PATH_MAX );
  std::string exePath = std::string( result, ( count > 0 ) ? static_cast<size_t>( count ) : 0 );
  size_t pos          = exePath.find_last_of( "\\/" );
  return ( std::string::npos == pos ) ? "" : exePath.substr( 0, pos );
#else
  char result[ PATH_MAX ];
  GetModuleFileName( NULL, result, 255 );
  std::string exePath = std::string( result );

  size_t pos = exePath.find_last_of( PATH_SPLIT );
  return exePath.substr( 0, pos );
#endif
}

static bool createPath( const std::string path )
{
  size_t start = 0, next = 0;
  if ( path.size() )
  {
    next = start = 1;
  }
  while ( path.size() > 1 || start == 0 )
  {
    if ( next == std::string::npos )
    {
      break;
    }
    next             = path.find( PATH_SPLIT, start );
    auto parent_path = path.substr( 0, next );

    if ( dirAccess( parent_path.c_str() ) == 0 )
    {
      start = next + strlen( PATH_SPLIT );
      continue;
    }
#ifdef __linux
    if ( mkdir( parent_path.c_str(), S_IRWXU | S_IRWXG ) != 0 )
#endif

#ifdef _WIN32
      if ( mkdir( parent_path.c_str() ) != 0 )
#endif
      {
        if ( dirAccess( parent_path.c_str() ) == 0 )
        {
          continue;
        }
        return false;
      }

    if ( next == std::string::npos )
    {
      break;
    }
    start = next + strlen( PATH_SPLIT );
  }
  return true;
}

std::string getTextVal( const std::string &path )
{
  std::ifstream f( path );
  if ( !f.is_open() )
  {
    return "";
  }

  std::stringstream ss;
  ss << f.rdbuf();

  return ss.str();
}

#ifdef __linux
static std::string expand_user( std::string path )
{
  if ( !path.empty() && path[ 0 ] == '~' )
  {
    if ( path.size() == 1 || path[ 1 ] == '/' )
    {
      // or other error handling
      char const *home = getenv( "HOME" );
      if ( home || ( home = getenv( "USERPROFILE" ) ) )
      {
        path.replace( 0, 1, home );
      }
      else
      {
        char const *hdrive = getenv( "HOMEDRIVE" ), *hpath = getenv( "HOMEPATH" );
        if ( hdrive && hpath )
        {
          path.replace( 0, 1, std::string( hdrive ) + hpath );
        }
      }
    }
  }
  return path;
}
#endif

void write2file( const std::string &file_path, const std::string &context )
{
  FILE *file = nullptr;

  file = fopen( file_path.c_str(), "wb" );
  fseek( file, 0, SEEK_SET );

  fwrite( context.c_str(), context.size(), 1, file );
  fclose( file );
}

#ifdef DEBUG
std::atomic_int g_intcout;
#endif
constexpr uint32_t max_cycle_size = 10 * 1024;

LogRecordInfo *new_log_data()
{
#ifdef DEBUG
  g_intcout.fetch_add( 1 );
#endif
  LogRecordInfo *data = nullptr;
  {
#ifdef USING_CYCLE
    std::lock_guard<std::mutex> lk( g_log_cycle_mutex );
    if ( g_log_cycle_deque.size() )
    {
      data = g_log_cycle_deque.front();
      g_log_cycle_deque.pop_front();
    }
#endif
  }
  if ( !data )
  {
    data = new LogRecordInfo();
  }
  return data;
}

void delete_log_data( LogRecordInfo *data )
{
#ifdef DEBUG
  auto ret = g_intcout.fetch_sub( 1 );
  DEBUG_LOG( --ret );
#endif
#ifdef USING_CYCLE
  if ( max_cycle_size < g_log_cycle_deque.size() )
  {
    std::lock_guard<std::mutex> lk( g_log_cycle_mutex );
    g_log_cycle_deque.push_back( data );
  }
  else
  {
    delete data;
  }
#else
  delete data;
#endif
}

LogRecordInfoPtr createLogRecord()
{
  // return std::make_shared<LogRecordInfo>();
  return std::shared_ptr<LogRecordInfo>( new_log_data(), delete_log_data );
}

LogRecordInfoPtr copyLogRecord( LogRecordInfoPtr log )
{
  auto new_log = createLogRecord();
  *new_log     = *log;
  return new_log;
}

void initModuleGenerator()
{
  RIGIST_RP_PROCESS_LOG_MODULE( "filter", ProcessLogCppFilter );
  RIGIST_RP_PROCESS_LOG_MODULE( "formatter", ProcessLogCppFormatter );
  RIGIST_RP_PROCESS_LOG_MODULE( "writer", ProcessLogCppSinkToFile );
  RIGIST_RP_PROCESS_LOG_MODULE( "asyncer", ProcessLogCppAsyncer );
  RIGIST_RP_PROCESS_LOG_MODULE( "printer", ProcessLogCppPrinter );
  RIGIST_RP_PROCESS_LOG_MODULE( "forker", ProcessLogCppForker );
};

INTERFACE_HIDED ProcessLogFramePtr ProcessLogFrame::instance_ = nullptr;

bool ProcessLogFrame::log( LogHelper &helper )
{
  return log( helper.key, helper.level, helper.text, helper.filename.c_str(), helper.line, helper.funcname.c_str() );
}

bool ProcessLogFrame::log( LogHelper &&helper )
{
  return log( std::move( helper.key ), helper.level, std::move( helper.text ), helper.filename.c_str(), helper.line,
              helper.funcname.c_str() );
}

ProcessLogFrame::ProcessLogFrame() {}

ProcessLogFrame::~ProcessLogFrame() {}

bool ProcessLogFrame::log( std::string key, LogLevel lv, std::string &&text, const char *filename, uint32_t line,
                           const char *funcname )
{
  if ( !checkInitialized() )
  {
    return false;
  }
  auto loginfo      = createLogRecord();
  loginfo->key      = key;
  loginfo->level    = lv;
  loginfo->time     = std::chrono::system_clock::now();
  loginfo->filename = filename;
  loginfo->line     = line;
  loginfo->log      = std::move( text );
  loginfo->funcname = funcname;

  if ( modules_.size() )
  {
    modules_[ 0 ]->process( loginfo );
  }
  return true;
  // return log( std::move(key), lv, text, filename, line );
}

bool ProcessLogFrame::log( std::string key, LogLevel lv, std::string &text, const char *filename, uint32_t line,
                           const char *funcname )
{
  if ( !checkInitialized() )
  {
    return false;
  }
  auto loginfo      = createLogRecord();
  loginfo->key      = key;
  loginfo->level    = lv;
  loginfo->time     = std::chrono::system_clock::now();
  loginfo->filename = filename;
  loginfo->line     = line;
  loginfo->log      = text;
  loginfo->funcname = funcname;

  if ( modules_.size() )
  {
    modules_[ 0 ]->process( loginfo );
  }
  return true;
}

#define JSON_GET_VALUE( j, t, d ) j.is##t() ? j.as##t() : d

#ifdef _WIN32

int vasprintf( char **ptr, const char *format, va_list ap )
{
  int len;

  len  = _vscprintf_p( format, ap ) + 1;
  *ptr = (char *)malloc( len * sizeof( char ) );
  if ( !*ptr )
  {
    return -1;
  }

  return _vsprintf_p( *ptr, len, format, ap );
}
#endif

bool ProcessLogFrame::logf( std::string key, LogLevel lv, const char *filename, uint32_t line, const char *funcname,
                            const char *fmt, ... )
{
  if ( !checkInitialized() )
  {
    return false;
  }
  char *pbuff = nullptr;
  va_list args;
  va_start( args, fmt );
  auto vret = vasprintf( &pbuff, fmt, args );
  va_end( args );
  UNUSE( vret );
  bool ret = false;

  if ( pbuff )
  {
    ret = log( std::move( key ), lv, pbuff, filename, line, funcname );
    // delete [] pbuff;
    free( pbuff );
  }

  return ret;
}

bool ProcessLogFrame::checkInitialized()
{
  if ( !inited_ )
  {
    std::string path;
#ifdef _WIN32
    static constexpr int buff_size = 255;
    char temp[ buff_size ]         = { 0 };
    memset( temp, 0, buff_size );

    GetEnvironmentVariable( "phdfilespath", temp, buff_size );
    // DEBUG_LOG("checkInitialized:"<<temp);
    if ( !strlen( temp ) )
    {
      GetEnvironmentVariable( "userprofile", temp, buff_size );
      path = temp;
      path += "/.rp";
    }
    else
    {
      path = temp;
    }

#endif

#ifdef __linux
    path = "~/.rp";
#endif
    auto module_name = getExeName();
    if ( module_name.size() )
    {
      module_name = module_name.substr( module_name.find_last_of( PATH_SPLIT ) + 1 );
    }
#ifdef __linux
    path = expand_user( path );
#endif
    if ( dirAccess( path.c_str() ) != 0 )
    {
      if ( !createPath( path ) )
      {
        return false;
      }
    }

    path = completePath( path );
    path += RP_LOG_CFG;

    if ( FileAccess( path.c_str() ) == 0 )
    {
      auto cfg = getTextVal( path );
      init( cfg.c_str(), module_name.c_str() );
      return inited_;
    }
    else
    {
      std::string cfg = R"(
                  {

                              "modules": [
                                      {"name":"formatter", "module":"formatter", "config":[
                                          {"key":"show_info", "val":"level"},
                                          {"key":"show_info", "val":"time"},
                                          {"key":"show_info", "val":"module"},
                                          {"key":"show_info", "val":"function"}
                                      ]},
                                      {"name":"filter", "module":"filter", "config":[
                                          {"key":"level", "val":"INFO"},
                                          {"key":"key", "val":""}
                                      ]},
                                      {"name":"printer", "module":"printer", "config":[
                                      ]},
                                      {"name":"writer", "module":"writer", "config":[)"
#ifdef _WIN32
                        R"({"key":"path", "val" : "C:\\rp_logs" },)"
#endif
#ifdef __linux
                        R"({"key":"path", "val" : "~/.rp/logs" },)"
#endif
                        R"({"key":"suffix", "val" : "info"}
                                      ]}
                                    ],
                              "links":[

                                {"module":"filter", "next":[
                                    {"module":"formatter", "type":0}
                                ]},
                                {"module":"formatter", "next":[
                                    {"module":"printer", "type":0},
                                    {"module":"writer", "type":0}
                                ]}
                                ]
                   }
                  )";

      write2file( path, cfg );
      init( cfg.c_str(), module_name.c_str() );
    }
  }

  return inited_;
}

bool ProcessLogFrame::init( const char *json_cfg, const char *module_name )
{
  std::string plugin_path;
  Json::Reader json_reader;
  Json::Value json;

  process_module_name_ = module_name;
  initModuleGenerator();

  if ( !json_reader.parse( json_cfg, json ) )
  {
    return false;
  }

  auto &json_modules = json[ "modules" ];

  if ( !json_modules.isArray() )
  {
    return false;
  }

  for ( auto &json_module : json_modules )
  {
    if ( !json_module.isObject() )
    {
      continue;
    }

    std::string name   = JSON_GET_VALUE( json_module[ "name" ], String, "" );
    std::string module = JSON_GET_VALUE( json_module[ "module" ], String, "" );

    if ( module.empty() )
    {
      continue;
    }
    if ( !g_module_generators.count( module ) )
    {
      continue;
    }
    auto plugin       = g_module_generators[ module ]( process_module_name_ );
    auto &json_config = json_module[ "config" ];
    if ( json_config.isArray() )
    {
      for ( auto &item : json_config )
      {
        std::string key = JSON_GET_VALUE( item[ "key" ], String, "" );
        std::string val = JSON_GET_VALUE( item[ "val" ], String, "" );
        if ( key.empty() )
        {
          continue;
        }
        plugin->setConfig( key.c_str(), val.c_str() );
      }
    }

    plugin->init( module.c_str() );
    modules_map_[ name ] = plugin;
  }

  auto &json_link = json[ "links" ];
  if ( !json_link.isArray() )
  {
    return false;
  }

  for ( auto &item : json_link )
  {
    if ( !item.isObject() )
    {
      continue;
    }

    std::string module_name = JSON_GET_VALUE( item[ "module" ], String, "" );
    uint32_t refcout        = 0;
    if ( modules_map_.count( module_name ) )
    {
      std::vector<std::pair<ProcessLogCppBasePtr, int>> link_infoes;
      auto &next_modules = item[ "next" ];

      if ( next_modules.isArray() )
      {
        for ( auto &next_one : next_modules )
        {
          if ( !next_one.isObject() )
          {
            continue;
          }
          auto next_module_name = JSON_GET_VALUE( next_one[ "module" ], String, "" );

          if ( !modules_map_.count( next_module_name ) )
          {
            continue;
          }

          auto next_module = modules_map_[ next_module_name ];
          auto type        = JSON_GET_VALUE( next_one[ "type" ], Int, 0 );

          refcout += (uint32_t)( type == 0 );
          link_infoes.push_back( { next_module, JSON_GET_VALUE( next_one[ "type" ], Int, 0 ) } );
        }
      }

      auto module = modules_map_[ module_name ];

      module->addNextHandle( "",
                             [ = ]( LogRecordInfoPtr info )
                             {
                               if ( info )
                               {
                                 for ( auto next_model_name : link_infoes )
                                 {
                                   auto info_tmp = info;
                                   if ( next_model_name.second == 1 )
                                   {
                                     info_tmp = copyLogRecord( info_tmp );
                                   }
                                   next_model_name.first->process( info_tmp );
                                 }
                               }
                             } );
      modules_.push_back( module );
    }
  }

  inited_ = true;
  return true;
}

}  // namespace rp_process_log
